@@ -142,18 +142,31 @@ gnome_print_dialog_init (GnomePrintDialog *gpd)
 }
 
 static void
-gpd_copies_set (GnomePrintCopiesSelector *gpc, gint copies, gboolean collate, GnomePrintDialog *gpd)
+gpd_copies_set (GnomePrintCopiesSelector *gpc, 
+		gint copies,
+		GnomePrintDialog *gpd)
 {
-	if (gpd->config) {
-		gnome_print_config_set_int (gpd->config, GNOME_PRINT_KEY_NUM_COPIES, copies);
-		gnome_print_config_set_boolean (gpd->config, GNOME_PRINT_KEY_COLLATE, collate);
-	}
+	if (gpd->config)
+		gnome_print_config_set_int (gpd->config, 
+					    GNOME_PRINT_KEY_NUM_COPIES, 
+					    copies);
+}
+
+static void
+gpd_collate_set (GnomePrintCopiesSelector *gpc, 
+		 gboolean collate, 
+		 GnomePrintDialog *gpd)
+{
+	if (gpd->config)
+		gnome_print_config_set_boolean (gpd->config, 
+						GNOME_PRINT_KEY_COLLATE, 
+						collate);
 }
 
 static GtkWidget *
 gpd_create_job_page (GnomePrintDialog *gpd)
 {
-	GtkWidget *hb, *vb, *f, *c;
+	GtkWidget *hb, *vb, *f, *c, *l;
 
 	hb = gtk_hbox_new (FALSE, 0);
 
@@ -162,14 +175,23 @@ gpd_create_job_page (GnomePrintDialog *gpd)
 
 	gtk_box_pack_start (GTK_BOX (hb), vb, FALSE, FALSE, 0);
 
-	f = gtk_frame_new (_("Print range"));
+	f = gtk_frame_new ("");
+	gtk_frame_set_shadow_type (GTK_FRAME (f), GTK_SHADOW_NONE);
+	l = gtk_label_new ("");
+	gtk_label_set_markup (GTK_LABEL (l), _("<b>Print range</b>"));
+	gtk_frame_set_label_widget (GTK_FRAME (f), l);
+	gtk_widget_show (l);
 	gtk_widget_hide (f);
 	gtk_box_pack_start (GTK_BOX (vb), f, FALSE, FALSE, 0);
 	g_object_set_data (G_OBJECT (hb), "range", f);
 
 	c = gnome_print_copies_selector_new ();
-	if (gpd)
-		g_signal_connect (G_OBJECT (c), "copies_set", (GCallback) gpd_copies_set, gpd);
+	if (gpd) {
+		g_signal_connect (G_OBJECT (c), "copies_set", 
+				  (GCallback) gpd_copies_set, gpd);
+		g_signal_connect (G_OBJECT (c), "collate_set", 
+				  (GCallback) gpd_collate_set, gpd);
+	}
 	gtk_widget_hide (c);
 	gtk_box_pack_start (GTK_BOX (vb), c, FALSE, FALSE, 0);
 	g_object_set_data (G_OBJECT (hb), "copies", c);
@@ -185,6 +207,7 @@ gpd_create_range (gint flags, GtkWidget *range, const guchar *clabel, const guch
 	gint row;
 
 	t = gtk_table_new (4, 2, FALSE);
+	gtk_container_set_border_width (GTK_CONTAINER (t), 6);
 
 	group = NULL;
 	row = 0;
@@ -302,6 +325,19 @@ gnome_print_dialog_construct (GnomePrintDialog *gpd, const guchar *title, gint f
 		l = gtk_label_new_with_mnemonic (_("Job"));
 		gtk_widget_show (l);
 		gtk_notebook_append_page (GTK_NOTEBOOK (gpd->notebook), gpd->job, l);
+
+		{
+			int copies;
+			gboolean collate;
+
+			if (!gnome_print_config_get_int (gpd->config, 
+			        GNOME_PRINT_KEY_NUM_COPIES, &copies))
+				copies = 1;
+			if (!gnome_print_config_get_boolean (gpd->config, 
+				GNOME_PRINT_KEY_COLLATE, &collate))
+				collate = FALSE;
+			gnome_print_dialog_set_copies (gpd, copies, collate);
+		}
 		
 		/* Add the printers page */
 		hb = gtk_hbox_new (FALSE, 0);
@@ -335,7 +371,11 @@ gnome_print_dialog_construct (GnomePrintDialog *gpd, const guchar *title, gint f
 		gtk_box_pack_start (GTK_BOX (GTK_DIALOG (gpd)->vbox), label, TRUE, TRUE, 0);
 	}
 
-	gtk_window_set_title (GTK_WINDOW (gpd), (title) ? title : (const guchar *) _("Gnome Print Dialog"));
+	gtk_window_set_title (GTK_WINDOW (gpd), 
+			      (title) ? title : 
+			      (const guchar *) _("Gnome Print Dialog"));
+
+	gtk_dialog_set_has_separator (GTK_DIALOG (gpd), FALSE);
 
 	gtk_dialog_add_buttons (GTK_DIALOG (gpd),
 				GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
@@ -488,6 +528,7 @@ gnome_print_dialog_construct_range_page (GnomePrintDialog *gpd, gint flags, gint
 		a = gtk_adjustment_new (start, start, end, 1, 10, 10);
 		g_object_set_data (G_OBJECT (hbox), "from", a);
 		sb = gtk_spin_button_new (GTK_ADJUSTMENT (a), 1, 0.0);
+		gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (sb), TRUE);
 		gtk_widget_show (sb);
 		gtk_box_pack_start (GTK_BOX (hbox), sb, FALSE, FALSE, 0);
 		gtk_label_set_mnemonic_widget ((GtkLabel *) l, sb);
@@ -512,6 +553,7 @@ gnome_print_dialog_construct_range_page (GnomePrintDialog *gpd, gint flags, gint
 		a = gtk_adjustment_new (end, start, end, 1, 10, 10);
 		g_object_set_data (G_OBJECT (hbox), "to", a);
 		sb = gtk_spin_button_new (GTK_ADJUSTMENT (a), 1, 0.0);
+		gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (sb), TRUE);
 		gtk_widget_show (sb);
 		gtk_box_pack_start (GTK_BOX (hbox), sb, FALSE, FALSE, 0);
 		gtk_label_set_mnemonic_widget ((GtkLabel *) l, sb);
@@ -633,7 +675,7 @@ gnome_print_dialog_get_range_page (GnomePrintDialog *gpd, gint *start, gint *end
  * copies indicator, then a default of 1 copy is returned.
  **/
 void
-gnome_print_dialog_get_copies (GnomePrintDialog *gpd, gint *copies, gint *collate)
+gnome_print_dialog_get_copies (GnomePrintDialog *gpd, gint *copies, gboolean *collate)
 {
 	g_return_if_fail (gpd != NULL);
 	g_return_if_fail (GNOME_IS_PRINT_DIALOG (gpd));
@@ -663,7 +705,7 @@ gnome_print_dialog_get_copies (GnomePrintDialog *gpd, gint *copies, gint *collat
  * Sets the print copies and collation status in the print dialogue.
  **/
 void
-gnome_print_dialog_set_copies (GnomePrintDialog *gpd, gint copies, gint collate)
+gnome_print_dialog_set_copies (GnomePrintDialog *gpd, gint copies, gboolean collate)
 {
 	GnomePrintCopiesSelector *c;
 
