@@ -36,7 +36,7 @@ G_BEGIN_DECLS
 #define GNOME_PRINT_DIALOG(o)           (G_TYPE_CHECK_INSTANCE_CAST ((o), GNOME_TYPE_PRINT_DIALOG, GnomePrintDialog))
 #define GNOME_PRINT_DIALOG_CLASS(k)     (G_TYPE_CHECK_CLASS_CAST    ((k), GNOME_TYPE_PRINT_DIALOG, GnomePrintDialogClass))
 #define GNOME_IS_PRINT_DIALOG(o)        (G_TYPE_CHECK_INSTANCE_TYPE ((o), GNOME_TYPE_PRINT_DIALOG))
-#define GNOME_IS_PRINT_DIALOG_CLASS(k)  (G_TYPE_CHECK_CLASS_TYPE    ((o), GNOME_TYPE_PRINT_DIALOG))
+#define GNOME_IS_PRINT_DIALOG_CLASS(k)  (G_TYPE_CHECK_CLASS_TYPE    ((k), GNOME_TYPE_PRINT_DIALOG))
 #define GNOME_PRINT_DIALOG_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS  ((o), GNOME_TYPE_PRINT_DIALOG), GnomePrintDialogClass)
 
 typedef struct _GnomePrintDialog      GnomePrintDialog;
@@ -99,7 +99,7 @@ void gnome_print_dialog_construct_range_page (GnomePrintDialog *gpd, gint flags,
 					      gint start, gint end,
 					      const guchar *currentlabel, const guchar *rangelabel);
 
-void                gnome_print_dialog_get_copies (GnomePrintDialog *gpd, gint *copies, gint *collate);
+void                gnome_print_dialog_get_copies (GnomePrintDialog *gpd, gint *copies, gboolean *collate);
 void                gnome_print_dialog_set_copies (GnomePrintDialog *gpd, gint copies, gint collate);
 GnomePrintConfig *  gnome_print_dialog_get_config (GnomePrintDialog *gpd);
 GnomePrintRangeType gnome_print_dialog_get_range  (GnomePrintDialog *gpd);
