@@ -43,13 +43,15 @@
 /* This two have to go away */
 #include "gnome-print-paper-preview.h"
 #include "gpaui/gpa-paper-preview-item.h"
+
 #include "gpaui/gpa-option-menu.h"
+#include "gpaui/gpa-spinbutton.h"
 
 #define MM(v) ((v) * 72.0 / 25.4)
 #define CM(v) ((v) * 72.0 / 2.54)
 #define M(v)  ((v) * 72.0 / 0.0254)
 
-#define PAD 4
+#define PAD 6
 
 /*
  * GnomePaperSelector widget
@@ -58,21 +60,19 @@
 struct _GnomePaperSelector {
 	GtkHBox box;
 
-	guint updating : 1;
-
 	GnomePrintConfig *config;
 	gint flags;
 
 	GtkWidget *preview;
 
-	GtkWidget *pmenu, *pomenu, *lomenu, *lymenu;
-	GtkWidget *pw, *ph, *us;
-	GtkWidget *pf;
-	gdouble w, h;
+	GtkWidget *pmenu, *pomenu, *lomenu, *lymenu, *trmenu;
+	GnomePrintUnitSelector *us;
 
-	GtkWidget *margin_frame;
-	GtkSpinButton *margin_top,  *margin_bottom, *margin_left, *margin_right;
-	gdouble mt, mb, ml, mr;
+	gdouble mt, mb, ml, mr; /* Values for margins   */
+	gdouble pw, ph;         /* Values for page size */
+
+	struct {GPASpinbutton *t, *b, *l, *r;} m; /* Spins for margins   */
+	struct {GPASpinbutton *w, *h;} p;         /* Spins for page size */
 
 	GPANode *paper_size_node;
 	GPANode *printer;
@@ -125,128 +125,57 @@ gnome_paper_selector_class_init (GnomePaperSelectorClass *klass)
 	object_class->finalize = gnome_paper_selector_finalize;
 }
 
-typedef struct {
-	guchar *id, *name;
-	gdouble affine[6];
-} GPPOrientation;
+static void
+gnome_paper_selector_update_spin_limits (GnomePaperSelector *ps)
+{
+	g_return_if_fail (GNOME_IS_PAPER_SELECTOR (ps));
 
-static const GPPOrientation porient[] = {
-	{"R0", "Straight", {1, 0, 0, 1, 0, 0}},
-	{"R90", "Rotated 90 degrees", {0, -1, 1, 0, 0, 1}},
-	{"R180", "Rotated 180 degrees", {-1, 0, 0, -1, 1, 1}},
-	{"R270", "Rotated 270 degrees", {0, 1, -1, 0, 1, 0}}
-};
-
-static const GPPOrientation lorient[] = {
-	{"R0", "Portrait", {1, 0, 0, 1, 0, 0}},
-	{"R90", "Landscape", {0, 1, -1, 0, 0, 1}},
-	{"R180", "Upside down portrait", {-1, 0, 0, -1, 1, 1}},
-	{"R270", "Upside down landscape", {0, -1, 1, 0, 1, 0}}
-};
-
-typedef struct {
-	guchar *id;
-	guchar *name;
-	gdouble width, height;
-	gint num_affines;
-	const gdouble *affines;
-} GPPLayout;
-
-static const gdouble lyid[] = {1, 0, 0, 1, 0, 0};
-static const gdouble ly21[] = {0, -0.707, 0.707, 0, 0, 1,
-			       0, -0.707, 0.707, 0, 0, 0.5};
-static const gdouble ly41[] = {0.5, 0, 0, 0.5, 0, 0.5,
-			       0.5, 0, 0, 0.5, 0.5, 0.5,
-			       0.5, 0, 0, 0.5, 0, 0,
-			       0.5, 0, 0, 0.5, 0.5, 0};
-static const gdouble ly2[] = {0, -1, 1, 0, 0, 1,
-			       0, -1, 1, 0, 0, 0.5};
-static const gdouble ly2f[] = {0, -1, 1, 0, 0, 0.5,
-			       0, 1, -1, 0, 1, 0.5};
-
-static const GPPLayout layout[] = {
-	{"Plain", "Plain", 1, 1, 1, lyid},
-	{"2_1", "2 Pages to 1", 0.5, 1, 2, ly21},
-	{"4_1", "4 Pages to 1", 0.5, 0.5, 4, ly41},
-	{"I2_1", "Divided", 0.5, 1, 2, ly2},
-	{"IM2_1", "Folded", 0.5, 1, 2, ly2f}
-};
-#define NUM_LAYOUTS (sizeof (layout) / sizeof (layout[0]))
+	/*
+	 * Margins cannot be larger than the page size minus the
+	 * opposite margin
+	 */
+	ps->m.t->upper = ps->ph - ps->mb; gpa_spinbutton_update (ps->m.t);
+	ps->m.b->upper = ps->ph - ps->mt; gpa_spinbutton_update (ps->m.b);
+	ps->m.r->upper = ps->pw - ps->ml; gpa_spinbutton_update (ps->m.r);
+	ps->m.l->upper = ps->pw - ps->mr; gpa_spinbutton_update (ps->m.l);
+}
 
 static void
 gps_psize_value_changed (GtkAdjustment *adj, GnomePaperSelector *ps)
 {
-	const GnomePrintUnit *unit;
-	gdouble w, h, max_wh;
+	g_return_if_fail (GNOME_IS_PAPER_SELECTOR (ps));
 
-	if (ps->updating)
-		return;
+	/* Did the value really change? */
+	if ((fabs (ps->pw - ps->p.w->value) < 0.1) &&
+	    (fabs (ps->ph - ps->p.h->value) < 0.1))
+		return; 
+	ps->pw = ps->p.w->value;
+	ps->ph = ps->p.h->value;
 
-	unit = gnome_print_unit_selector_get_unit (GNOME_PRINT_UNIT_SELECTOR (ps->us));
-	w = gtk_spin_button_get_value (GTK_SPIN_BUTTON (ps->pw));
-
-	gnome_print_convert_distance (&w, unit, GNOME_PRINT_PS_UNIT);
-	h = gtk_spin_button_get_value (GTK_SPIN_BUTTON (ps->ph));
-
-	gnome_print_convert_distance (&h, unit, GNOME_PRINT_PS_UNIT);
-
-	gnome_print_config_set_length (ps->config, GNOME_PRINT_KEY_PAPER_WIDTH,
-				       w, GNOME_PRINT_PS_UNIT);
-	gnome_print_config_set_length (ps->config, GNOME_PRINT_KEY_PAPER_HEIGHT,
-				       h, GNOME_PRINT_PS_UNIT);
-
-	if ((fabs (ps->w - w) < 0.1) && (fabs (ps->h - h) < 0.1))
-		return;
-
-	ps->w = w;
-	ps->h = h;
-	max_wh = MAX (w, h);
-	gtk_spin_button_get_adjustment (ps->margin_top)->upper = max_wh;
-	gtk_adjustment_changed (gtk_spin_button_get_adjustment (ps->margin_top));
-	gtk_spin_button_get_adjustment (ps->margin_bottom)->upper = max_wh;
-	gtk_adjustment_changed (gtk_spin_button_get_adjustment (ps->margin_bottom));
-	gtk_spin_button_get_adjustment (ps->margin_left)->upper = max_wh;
-	gtk_adjustment_changed (gtk_spin_button_get_adjustment (ps->margin_left));
-	gtk_spin_button_get_adjustment (ps->margin_right)->upper = max_wh;
-	gtk_adjustment_changed (gtk_spin_button_get_adjustment (ps->margin_right));
+	gnome_paper_selector_update_spin_limits (ps);
 }
 
 static void
 gps_m_size_value_changed (GtkAdjustment *adj, GnomePaperSelector *ps)
 {
-	const GnomePrintUnit *unit;
-	gdouble mt, mb, ml, mr;
+	g_return_if_fail (GNOME_IS_PAPER_SELECTOR (ps));
 
-	unit = gnome_print_unit_selector_get_unit (GNOME_PRINT_UNIT_SELECTOR (ps->us));
-
-	mt = gtk_spin_button_get_value (GTK_SPIN_BUTTON (ps->margin_top));
-	gnome_print_convert_distance (&mt, unit, GNOME_PRINT_PS_UNIT);
-	gnome_print_config_set_length (ps->config, GNOME_PRINT_KEY_PAGE_MARGIN_TOP, mt, GNOME_PRINT_PS_UNIT);
-
-	mb = gtk_spin_button_get_value (GTK_SPIN_BUTTON (ps->margin_bottom));
-	gnome_print_convert_distance (&mb, unit, GNOME_PRINT_PS_UNIT);
-	gnome_print_config_set_length (ps->config, GNOME_PRINT_KEY_PAGE_MARGIN_BOTTOM, mb, GNOME_PRINT_PS_UNIT);
-
-	ml = gtk_spin_button_get_value (GTK_SPIN_BUTTON (ps->margin_left));
-	gnome_print_convert_distance (&ml, unit, GNOME_PRINT_PS_UNIT);
-	gnome_print_config_set_length (ps->config, GNOME_PRINT_KEY_PAGE_MARGIN_LEFT, ml, GNOME_PRINT_PS_UNIT);
-
-	mr = gtk_spin_button_get_value (GTK_SPIN_BUTTON (ps->margin_right));
-	gnome_print_convert_distance (&mr, unit, GNOME_PRINT_PS_UNIT);
-	gnome_print_config_set_length (ps->config, GNOME_PRINT_KEY_PAGE_MARGIN_RIGHT, mr, GNOME_PRINT_PS_UNIT);
-
-	if ((fabs (ps->mt - mt) < 0.1) && (fabs (ps->mb - mb) < 0.1) &&
-	    (fabs (ps->ml - ml) < 0.1) && (fabs (ps->mr - mr) < 0.1))
-		return;
-
-	ps->ml = ml;
-	ps->mr = mr;
-	ps->mt = mt;
-	ps->mb = mb;
+	/* Did the value really change? */
+	if ((fabs (ps->mt - ps->m.t->value) < 0.1) &&
+	    (fabs (ps->mb - ps->m.b->value) < 0.1) &&
+	    (fabs (ps->ml - ps->m.l->value) < 0.1) &&
+	    (fabs (ps->mr - ps->m.r->value) < 0.1))
+		return; 
+	ps->ml = ps->m.l->value;
+	ps->mr = ps->m.r->value;
+	ps->mt = ps->m.t->value;
+	ps->mb = ps->m.b->value;
 
 	gpa_paper_preview_item_set_logical_margins 
-		(GNOME_CANVAS_ITEM (GNOME_PAPER_PREVIEW (ps->preview)->item),
-		 ml, mr, mt, mb);
+		(GPA_PAPER_PREVIEW_ITEM (GNOME_PAPER_PREVIEW 
+					 (ps->preview)->item),
+		 ps->ml, ps->mr, ps->mt, ps->mb);
+	gnome_paper_selector_update_spin_limits (ps);
 }
 
 static void
@@ -272,7 +201,6 @@ gnome_paper_selector_init (GnomePaperSelector *ps)
 {
 	ps->config = NULL;
 	ps->flags = 0;
-	ps->updating = FALSE;
 }
 
 static void
@@ -374,81 +302,46 @@ lmargin_unit_deactivated (GtkSpinButton *spin_button,
 	return FALSE;
 }
 
-typedef struct {
-	guchar *abbr;
-	gint digits;
-	gfloat step_increment;
-} GnomePrintPaperSelectorSpinProps_t ;
-
-static const GnomePrintPaperSelectorSpinProps_t gpps_spin_props[] = {
-	{N_("Pt"), 1, 1.0},
-	{N_("mm"), 1, 1.0},
-	{N_("cm"), 2, 0.5},
-	{N_("m"),  3, 0.01},
-	{N_("in"), 2, 0.25},
-	{NULL,     2, 1.0}          /* Default must be last */
-};
-
 static void
-gnome_paper_selector_spin_adapt_to_unit (GtkSpinButton *spin, const GnomePrintUnit *unit)
+gnome_print_paper_selector_update_spin_units (GnomePaperSelector *ps)
 {
-	gint num_of_units = sizeof (gpps_spin_props) / sizeof (GnomePrintPaperSelectorSpinProps_t);
-	gint i;
-	GtkAdjustment *adjustment;
+	const GnomePrintUnit *unit;
 
-	g_return_if_fail (GTK_IS_SPIN_BUTTON (spin));
-	adjustment = gtk_spin_button_get_adjustment (spin);
-	g_return_if_fail (GTK_IS_ADJUSTMENT (adjustment));
+	g_return_if_fail (GNOME_IS_PAPER_SELECTOR (ps));
 
-	for (i = 0; i < num_of_units; i++) {
-		if (gpps_spin_props[i].abbr == NULL ||
-		    !strcmp (unit->abbr, gpps_spin_props[i].abbr)) {
-			adjustment->step_increment = gpps_spin_props[i].step_increment;
-			adjustment->page_increment = gpps_spin_props[i].step_increment * 10;
-			gtk_adjustment_changed (adjustment);
-			gtk_spin_button_set_digits (spin, gpps_spin_props[i].digits);
-			return;
-		}
-	}
+	unit = gnome_print_unit_selector_get_unit (ps->us);
+	if (!unit) return;
+
+	gpa_spinbutton_set_unit (ps->m.t, unit->abbr);
+	gpa_spinbutton_set_unit (ps->m.b, unit->abbr);
+	gpa_spinbutton_set_unit (ps->m.r, unit->abbr);
+	gpa_spinbutton_set_unit (ps->m.l, unit->abbr);
+	gpa_spinbutton_set_unit (ps->p.h, unit->abbr);
+	gpa_spinbutton_set_unit (ps->p.w, unit->abbr);
 }
 
 static void
 gnome_paper_selector_unit_changed_cb (GnomePrintUnitSelector *sel, GnomePaperSelector *ps)
 {
-	const GnomePrintUnit *unit;
-
-	g_return_if_fail (ps != NULL);
-
-	unit = gnome_print_unit_selector_get_unit (sel);
-	if (unit) {
-		gnome_print_config_set (ps->config, GNOME_PRINT_KEY_PREFERED_UNIT, unit->abbr);
-		gnome_paper_selector_spin_adapt_to_unit (GTK_SPIN_BUTTON (ps->pw), unit);
-		gnome_paper_selector_spin_adapt_to_unit (GTK_SPIN_BUTTON (ps->ph), unit);
-		gnome_paper_selector_spin_adapt_to_unit (ps->margin_top, unit);
-		gnome_paper_selector_spin_adapt_to_unit (ps->margin_bottom, unit);
-		gnome_paper_selector_spin_adapt_to_unit (ps->margin_left, unit);
-		gnome_paper_selector_spin_adapt_to_unit (ps->margin_right, unit);
-	}
+	gnome_print_paper_selector_update_spin_units (ps);
 }
 
 static void
-gnome_paper_unit_selector_request_update_cb (GPANode *node, guint flags,  GnomePaperSelector *ps)
+gnome_paper_unit_selector_request_update_cb (GPANode *node, guint flags,
+					     GnomePaperSelector *ps)
 {
 	guchar *unit_txt;
 
-	if (ps->updating)
-		return;
-
 	unit_txt = gnome_print_config_get (ps->config, GNOME_PRINT_KEY_PREFERED_UNIT);
 	if (!unit_txt) {
 		g_warning ("Could not get GNOME_PRINT_KEY_PREFERED_UNIT");
 		return;
 	}
 
-	gnome_print_unit_selector_set_unit (GNOME_PRINT_UNIT_SELECTOR (ps->us),
-					    gnome_print_unit_get_by_abbreviation
-					    (unit_txt));
+	gnome_print_unit_selector_set_unit (ps->us,
+			gnome_print_unit_get_by_abbreviation (unit_txt));
 	g_free (unit_txt);
+	gnome_print_paper_selector_update_spin_units (ps);
 }
 
 /**
@@ -462,33 +355,19 @@ gnome_paper_unit_selector_request_update_cb (GPANode *node, guint flags,  GnomeP
 static void
 gnome_paper_selector_paper_size_modified_cb (GPANode *node, guint flags, GnomePaperSelector *ps)
 {
-	const GnomePrintUnit *unit, *to;
 	guchar *id;
 	gboolean sensitivity = FALSE;
-	gdouble height, width;
-	gint retval = 0;
 
 	id = gnome_print_config_get (ps->config, GNOME_PRINT_KEY_PAPER_SIZE);
-	if (id && *id && !strcmp (id, "Custom"))
-		sensitivity = TRUE;
-	
-	gtk_widget_set_sensitive (ps->pw, sensitivity);
-	gtk_widget_set_sensitive (ps->ph, sensitivity);
-
-	to = gnome_print_unit_selector_get_unit (GNOME_PRINT_UNIT_SELECTOR (ps->us));
-	height = 0.0;
-	width  = 0.0;
-	retval += gnome_print_config_get_length (ps->config, GNOME_PRINT_KEY_PAPER_HEIGHT, &height, &unit);
-	retval += gnome_print_config_get_length (ps->config, GNOME_PRINT_KEY_PAPER_WIDTH,  &width,  &unit);
-	g_return_if_fail (retval == 2 && to != NULL);
-	gnome_print_convert_distance (&width,  unit, to);
-	gnome_print_convert_distance (&height, unit, to);
-	ps->updating = TRUE;
-	gtk_spin_button_set_value (GTK_SPIN_BUTTON (ps->ph), height);
-	gtk_spin_button_set_value (GTK_SPIN_BUTTON (ps->pw), width);
-	ps->updating = FALSE;
-	if (id)
+	if (id) {
+		if (*id && !strcmp (id, "Custom")) sensitivity = TRUE;
 		g_free (id);
+	}
+
+	gtk_widget_set_sensitive (GTK_WIDGET (ps->p.w), sensitivity);
+	gtk_widget_set_sensitive (GTK_WIDGET (ps->p.h), sensitivity);
+
+	gnome_paper_selector_update_spin_limits (ps);
 }
 
 static void
@@ -527,12 +406,10 @@ gnome_paper_selector_printer_changed_cb (GPANode *node, guint flags, GnomePaperS
 static void
 gnome_paper_selector_construct (GnomePaperSelector *ps)
 {
-	GtkWidget *vb, *f, *t, *l;
+	GtkWidget *vb, *f, *t, *l, *s;
 	GtkWidget *margin_table, *margin_label;
-	gdouble ml, mr, mt, mb;
-	GtkObject *wa, *ha;
+	gdouble ml, mr, mt, mb, pw = 1., ph = 1.;
 	AtkObject *atko;
-	gdouble config_height, config_width, config_max;
 	GPANode *printer;
 
 	g_return_if_fail (ps != NULL);
@@ -541,7 +418,31 @@ gnome_paper_selector_construct (GnomePaperSelector *ps)
 	printer = gpa_node_get_child_from_path (GNOME_PRINT_CONFIG_NODE (ps->config), "Printer");
 	g_return_if_fail (printer != NULL);
 	ps->printer = printer;
-	
+
+	/* Fetch the current settings. */
+	ml = MM(10);
+	mr = MM(10);
+	mt = MM(10);
+	mb = MM(10);
+	gnome_print_config_get_length (ps->config,
+				GNOME_PRINT_KEY_PAGE_MARGIN_LEFT, &ml, NULL);
+	gnome_print_config_get_length (ps->config,
+				GNOME_PRINT_KEY_PAGE_MARGIN_RIGHT, &mr, NULL);
+	gnome_print_config_get_length (ps->config,
+				GNOME_PRINT_KEY_PAGE_MARGIN_TOP, &mt, NULL);
+	gnome_print_config_get_length (ps->config,
+				GNOME_PRINT_KEY_PAGE_MARGIN_BOTTOM, &mb, NULL);
+	ps->ml = ml;
+	ps->mr = mr;
+	ps->mt = mt;
+	ps->mb = mb;
+	gnome_print_config_get_length (ps->config,
+			GNOME_PRINT_KEY_PAPER_WIDTH, &pw, NULL);
+	gnome_print_config_get_length (ps->config,
+			GNOME_PRINT_KEY_PAPER_HEIGHT, &ph, NULL);
+	ps->ph = ph;
+	ps->pw = pw;
+
 	gtk_box_set_spacing (GTK_BOX (ps), PAD);
 
 	/* VBox for controls */
@@ -550,7 +451,12 @@ gnome_paper_selector_construct (GnomePaperSelector *ps)
 	gtk_box_pack_start (GTK_BOX (ps), vb, FALSE, FALSE, 0);
 
 	/* Create frame for selection menus */
-	f = gtk_frame_new (_("Paper and layout"));
+	f = gtk_frame_new ("");
+	gtk_frame_set_shadow_type (GTK_FRAME (f), GTK_SHADOW_NONE);
+	l = gtk_label_new ("");
+	gtk_label_set_markup (GTK_LABEL (l), _("<b>Paper and layout</b>"));
+	gtk_frame_set_label_widget (GTK_FRAME (f), l);
+	gtk_widget_show (l);
 	gtk_widget_show (f);
 	gtk_box_pack_start (GTK_BOX (vb), f, FALSE, FALSE, 0);
 
@@ -579,45 +485,36 @@ gnome_paper_selector_construct (GnomePaperSelector *ps)
 	gtk_widget_show (l);
 	gtk_misc_set_alignment (GTK_MISC (l), 1.0, 0.5);
 	gtk_table_attach_defaults (GTK_TABLE (t), l, 1, 2, 1, 2);
-
-	config_width = 1;
-	gnome_print_config_get_length (ps->config, GNOME_PRINT_KEY_PAPER_WIDTH,
-				       &config_width, NULL);
-	ps->w = config_width;
-	wa = gtk_adjustment_new (config_width, 0.0001, 10000, 1, 10, 10);
-	ps->pw = gtk_spin_button_new (GTK_ADJUSTMENT (wa), 1, 2);
-	gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (ps->pw), TRUE);
-	gtk_widget_show (ps->pw);
-	gtk_table_attach_defaults (GTK_TABLE (t), ps->pw, 2, 3, 1, 2);
-	gtk_label_set_mnemonic_widget ((GtkLabel *) l, ps->pw);
-	gps_set_labelled_by_relation (ps->pw, l);
+	s = gpa_spinbutton_new (ps->config, GNOME_PRINT_KEY_PAPER_WIDTH,
+		0.0001, 10000, 1, 10, 10, 1, 2);
+	ps->p.w = GPA_SPINBUTTON (s);
+	gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (ps->p.w->spinbutton),
+				     TRUE);
+	gtk_widget_show (s);
+	gtk_table_attach_defaults (GTK_TABLE (t), s, 2, 3, 1, 2);
+	gtk_label_set_mnemonic_widget ((GtkLabel *) l, ps->p.w->spinbutton);
+	gps_set_labelled_by_relation (ps->p.w->spinbutton, l);
 
 	/* Custom paper Height */
 	l = gtk_label_new_with_mnemonic (_("_Height:"));
 	gtk_widget_show (l);
 	gtk_misc_set_alignment (GTK_MISC (l), 1.0, 0.5);
 	gtk_table_attach_defaults (GTK_TABLE (t), l, 1, 2, 2, 3);
-
-	config_height = 1;
-	gnome_print_config_get_length (ps->config, GNOME_PRINT_KEY_PAPER_HEIGHT,
-				       &config_height, NULL);
-	ps->h = config_height;
-	ha = gtk_adjustment_new (config_height, 0.0001, 10000, 1, 10, 10);
-	ps->ph = gtk_spin_button_new (GTK_ADJUSTMENT (ha), 1, 2);
-	gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (ps->ph), TRUE);
-	gtk_widget_show (ps->ph);
-	gtk_table_attach_defaults (GTK_TABLE (t), ps->ph, 2, 3, 2, 3);
-	gtk_label_set_mnemonic_widget ((GtkLabel *) l, ps->ph);
-	gps_set_labelled_by_relation (ps->ph, l);
+	s = gpa_spinbutton_new (ps->config, GNOME_PRINT_KEY_PAPER_HEIGHT,
+			0.0001, 10000, 1, 10, 10, 1, 2);
+	ps->p.h = GPA_SPINBUTTON (s);
+	gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (ps->p.h->spinbutton),
+				     TRUE);
+	gtk_widget_show (s);
+	gtk_table_attach_defaults (GTK_TABLE (t), s, 2, 3, 2, 3);
+	gtk_label_set_mnemonic_widget ((GtkLabel *) l, ps->p.h->spinbutton);
+	gps_set_labelled_by_relation (ps->p.h->spinbutton, l);
 
 	/* Custom paper Unit selector */
-	ps->us = gnome_print_unit_selector_new (GNOME_PRINT_UNIT_ABSOLUTE);
-	gtk_table_attach_defaults (GTK_TABLE (t), ps->us, 3, 4, 1, 2);
-	gnome_print_unit_selector_add_adjustment (GNOME_PRINT_UNIT_SELECTOR (ps->us), GTK_ADJUSTMENT (wa));
-	gnome_print_unit_selector_add_adjustment (GNOME_PRINT_UNIT_SELECTOR (ps->us), GTK_ADJUSTMENT (ha));
-	g_signal_connect (G_OBJECT (wa), "value_changed", (GCallback) gps_psize_value_changed, ps);
-	g_signal_connect (G_OBJECT (ha), "value_changed", (GCallback) gps_psize_value_changed, ps);
-	atko = gtk_widget_get_accessible (ps->us);
+	s = gnome_print_unit_selector_new (GNOME_PRINT_UNIT_ABSOLUTE);
+	ps->us = GNOME_PRINT_UNIT_SELECTOR (s);
+	gtk_table_attach_defaults (GTK_TABLE (t), s, 3, 4, 1, 2);
+	atko = gtk_widget_get_accessible (s);
 	atk_object_set_name (atko, _("Metric selector"));
 	atk_object_set_description (atko, _("Specifies the metric to use when setting the width and height of the paper"));
 
@@ -660,69 +557,72 @@ gnome_paper_selector_construct (GnomePaperSelector *ps)
 	gtk_label_set_mnemonic_widget ((GtkLabel *) l, ps->lymenu);
 	gps_set_labelled_by_relation (ps->lymenu, l);
 
+	/* Paper source */
+	l = gtk_label_new_with_mnemonic (_("Paper _tray:"));
+	gtk_widget_show(l);
+	gtk_misc_set_alignment (GTK_MISC (l), 1.0, 0.5);
+	gtk_table_attach_defaults (GTK_TABLE (t), l, 0, 1, 6, 7);
+
+	ps->trmenu = gpa_option_menu_new (ps->config, GNOME_PRINT_KEY_PAPER_SOURCE);
+	gtk_widget_show(ps->trmenu);
+	gtk_table_attach_defaults (GTK_TABLE (t), ps->trmenu, 1, 4, 6, 7);
+	gtk_label_set_mnemonic_widget ((GtkLabel *) l, ps->trmenu);
+	gps_set_labelled_by_relation (ps->trmenu, l);
+
 	/* Preview frame */
-	ps->pf = gtk_frame_new (_("Preview"));
-	gtk_widget_show (ps->pf);
-	gtk_box_pack_start (GTK_BOX (ps), ps->pf, TRUE, TRUE, 0);
+	f = gtk_frame_new ("");
+	gtk_frame_set_shadow_type (GTK_FRAME (f), GTK_SHADOW_NONE);
+	l = gtk_label_new ("");
+	gtk_label_set_markup (GTK_LABEL (l), _("<b>Preview</b>"));
+	gtk_frame_set_label_widget (GTK_FRAME (f), l);
+	gtk_widget_show (l);
+	gtk_widget_show (f);
+	gtk_box_pack_start (GTK_BOX (ps), f, TRUE, TRUE, 0);
 
 	ps->preview = gnome_paper_preview_new (ps->config);
 	gtk_widget_set_size_request (ps->preview, 160, 160);
 	gtk_widget_show (ps->preview);
-	gtk_container_add (GTK_CONTAINER (ps->pf), ps->preview);
+	gtk_container_add (GTK_CONTAINER (f), ps->preview);
 
 	atko = gtk_widget_get_accessible (ps->preview);
 	atk_object_set_name (atko, _("Preview"));
 	atk_object_set_description (atko, _("Preview of the page size, orientation and layout"));
 
 	/* Margins */
-	ml = MM(10);
-	mr = MM(10);
-	mt = MM(10);
-	mb = MM(10);
-	
-	gnome_print_config_get_length (ps->config, GNOME_PRINT_KEY_PAGE_MARGIN_LEFT, &ml, NULL);
-	gnome_print_config_get_length (ps->config, GNOME_PRINT_KEY_PAGE_MARGIN_RIGHT, &mr, NULL);
-	gnome_print_config_get_length (ps->config, GNOME_PRINT_KEY_PAGE_MARGIN_TOP, &mt, NULL);
-	gnome_print_config_get_length (ps->config, GNOME_PRINT_KEY_PAGE_MARGIN_BOTTOM, &mb, NULL);
+	f = gtk_frame_new ("");
+	gtk_frame_set_shadow_type (GTK_FRAME (f), GTK_SHADOW_NONE);
+	l = gtk_label_new ("");
+	gtk_label_set_markup (GTK_LABEL (l), _("<b>Margins</b>"));
+	gtk_frame_set_label_widget (GTK_FRAME (f), l);
+	gtk_widget_show (l);
 
-	ps->ml = ml;
-	ps->mr = mr;
-	ps->mt = mt;
-	ps->mb = mb;
-
-	ps->margin_frame = gtk_frame_new (_("Margins"));
-	gtk_box_pack_start (GTK_BOX (ps), ps->margin_frame, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (ps), f, FALSE, FALSE, 0);
 	margin_table = gtk_table_new ( 8, 1, TRUE);
 	gtk_container_set_border_width  (GTK_CONTAINER (margin_table), 4);
 
-	config_max = MAX (config_height, config_width);
-	ps->margin_top = GTK_SPIN_BUTTON (gtk_spin_button_new
-					  (GTK_ADJUSTMENT (gtk_adjustment_new
-							   (mt, 0, config_max,
-							    1, 10, 10)),
-					   1, 2));
-	gtk_table_attach (GTK_TABLE (margin_table), GTK_WIDGET (ps->margin_top),
+	s = gpa_spinbutton_new (ps->config,
+		GNOME_PRINT_KEY_PAGE_MARGIN_TOP, 0., ps->ph,
+		1., 10., 10., 1., 2.);
+	ps->m.t = GPA_SPINBUTTON (s);
+	gtk_table_attach (GTK_TABLE (margin_table), s,
 			  0, 1, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
-	ps->margin_bottom = GTK_SPIN_BUTTON (gtk_spin_button_new
-					     (GTK_ADJUSTMENT (gtk_adjustment_new
-							      (mb, 0, config_max,
-							       1, 10, 10)),
-					      1, 2));
-	gtk_table_attach (GTK_TABLE (margin_table), GTK_WIDGET (ps->margin_bottom),
+	s = gpa_spinbutton_new (ps->config,
+		GNOME_PRINT_KEY_PAGE_MARGIN_BOTTOM, 0., ps->ph,
+		1, 10, 10, 1., 2.);
+	ps->m.b = GPA_SPINBUTTON (s);
+	gtk_table_attach (GTK_TABLE (margin_table), s,
 			   0, 1, 7, 8,  GTK_FILL, GTK_FILL, 0, 0);
-	ps->margin_left = GTK_SPIN_BUTTON (gtk_spin_button_new
-					   (GTK_ADJUSTMENT (gtk_adjustment_new
-							    (ml, 0, config_max,
-							     1, 10, 10)),
-					    1, 2));
-	gtk_table_attach (GTK_TABLE (margin_table), GTK_WIDGET (ps->margin_left),
+	s = gpa_spinbutton_new (ps->config,
+		GNOME_PRINT_KEY_PAGE_MARGIN_LEFT, 0., ps->pw,
+		1, 10, 10, 1., 2.);
+	ps->m.l = GPA_SPINBUTTON (s);
+	gtk_table_attach (GTK_TABLE (margin_table), s,
 			  0, 1, 3, 4, GTK_FILL, GTK_FILL, 0, 0);
-	ps->margin_right = GTK_SPIN_BUTTON (gtk_spin_button_new
-					    (GTK_ADJUSTMENT (gtk_adjustment_new
-							     (mr, 0, config_max,
-							      1, 10, 10)),
-					     1, 2));
-	gtk_table_attach (GTK_TABLE (margin_table), GTK_WIDGET (ps->margin_right),
+	s = gpa_spinbutton_new (ps->config,
+		GNOME_PRINT_KEY_PAGE_MARGIN_RIGHT, 0., ps->pw,
+		1, 10, 10, 1., 2.);
+	ps->m.r = GPA_SPINBUTTON (s);
+	gtk_table_attach (GTK_TABLE (margin_table), s, 
 			  0, 1, 5, 6, GTK_FILL, GTK_FILL, 0, 0);
 	margin_label = gtk_label_new (_("Top"));
 	gtk_table_attach (GTK_TABLE (margin_table), margin_label,
@@ -736,58 +636,60 @@ gnome_paper_selector_construct (GnomePaperSelector *ps)
 	margin_label = gtk_label_new (_("Right"));
 	gtk_table_attach (GTK_TABLE (margin_table), margin_label,
 			  0, 1, 4, 5, GTK_FILL, GTK_FILL, 0, 0);
-
-	gtk_container_add (GTK_CONTAINER (ps->margin_frame), margin_table);
-	gnome_print_unit_selector_add_adjustment (GNOME_PRINT_UNIT_SELECTOR (ps->us),
-						  gtk_spin_button_get_adjustment (ps->margin_top));
-	gnome_print_unit_selector_add_adjustment (GNOME_PRINT_UNIT_SELECTOR (ps->us),
-						  gtk_spin_button_get_adjustment (ps->margin_bottom));
-	gnome_print_unit_selector_add_adjustment (GNOME_PRINT_UNIT_SELECTOR (ps->us),
-						  gtk_spin_button_get_adjustment (ps->margin_left));
-	gnome_print_unit_selector_add_adjustment (GNOME_PRINT_UNIT_SELECTOR (ps->us),
-						  gtk_spin_button_get_adjustment (ps->margin_right));
-	g_signal_connect (G_OBJECT (gtk_spin_button_get_adjustment (ps->margin_top)),
-			  "value_changed", G_CALLBACK (gps_m_size_value_changed), ps);
-	g_signal_connect (G_OBJECT (gtk_spin_button_get_adjustment (ps->margin_bottom)),
-			  "value_changed", G_CALLBACK (gps_m_size_value_changed), ps);
-	g_signal_connect (G_OBJECT (gtk_spin_button_get_adjustment (ps->margin_left)),
-			  "value_changed", G_CALLBACK (gps_m_size_value_changed), ps);
-	g_signal_connect (G_OBJECT (gtk_spin_button_get_adjustment (ps->margin_right)),
-			  "value_changed", G_CALLBACK (gps_m_size_value_changed), ps);
-	g_signal_connect (G_OBJECT (ps->margin_top), "focus_in_event",
-			  G_CALLBACK (lmargin_top_unit_activated), ps);
-	g_signal_connect (G_OBJECT (ps->margin_top), "focus_out_event",
-			  G_CALLBACK (lmargin_unit_deactivated), ps);
-	g_signal_connect (G_OBJECT (ps->margin_left), "focus_in_event",
-			  G_CALLBACK (lmargin_left_unit_activated), ps);
-	g_signal_connect (G_OBJECT (ps->margin_left), "focus_out_event",
-			  G_CALLBACK (lmargin_unit_deactivated), ps);
-	g_signal_connect (G_OBJECT (ps->margin_right), "focus_in_event",
-			  G_CALLBACK (lmargin_right_unit_activated), ps);
-	g_signal_connect (G_OBJECT (ps->margin_right), "focus_out_event",
-			  G_CALLBACK (lmargin_unit_deactivated), ps);
-	g_signal_connect (G_OBJECT (ps->margin_bottom), "focus_in_event",
-			  G_CALLBACK (lmargin_bottom_unit_activated), ps);
-	g_signal_connect (G_OBJECT (ps->margin_bottom), "focus_out_event",
-			  G_CALLBACK (lmargin_unit_deactivated), ps);
-
+	gtk_container_add (GTK_CONTAINER (f), margin_table);
 	if (ps->flags & GNOME_PAPER_SELECTOR_MARGINS)
-		gtk_widget_show_all (ps->margin_frame);
+		gtk_widget_show_all (f);
 
 	/* Hook up the Custom size widgets */
 	gnome_paper_selector_hook_paper_size (ps);
-	ps->handler_printer = g_signal_connect (G_OBJECT (ps->printer), "modified",
-						(GCallback) gnome_paper_selector_printer_changed_cb, ps);
-	
+	ps->handler_printer = g_signal_connect (G_OBJECT (ps->printer),
+		"modified", (GCallback) gnome_paper_selector_printer_changed_cb, ps);
+
 	/* Setup the unit selector */
 	gnome_paper_unit_selector_request_update_cb (NULL, 0, ps);
 	g_signal_connect (G_OBJECT (ps->us), "modified",
-			  G_CALLBACK (gnome_paper_selector_unit_changed_cb), ps);
+		G_CALLBACK (gnome_paper_selector_unit_changed_cb), ps);
 	ps->handler_unit = g_signal_connect (
 		G_OBJECT (gnome_print_config_get_node (ps->config)), "modified",
 		G_CALLBACK (gnome_paper_unit_selector_request_update_cb), ps);
 
-	gtk_widget_show (ps->us);
+	/* Connect signals */
+	g_signal_connect (
+		G_OBJECT (GTK_SPIN_BUTTON (ps->p.w->spinbutton)->adjustment),
+		"value_changed", G_CALLBACK (gps_psize_value_changed), ps);
+	g_signal_connect (
+		G_OBJECT (GTK_SPIN_BUTTON (ps->p.h->spinbutton)->adjustment), 
+		"value_changed", G_CALLBACK (gps_psize_value_changed), ps);
+	g_signal_connect (
+		G_OBJECT (GTK_SPIN_BUTTON (ps->m.t->spinbutton)->adjustment),
+		"value_changed", G_CALLBACK (gps_m_size_value_changed), ps);
+	g_signal_connect (
+		G_OBJECT (GTK_SPIN_BUTTON (ps->m.b->spinbutton)->adjustment),
+		"value_changed", G_CALLBACK (gps_m_size_value_changed), ps);
+	g_signal_connect (
+		G_OBJECT (GTK_SPIN_BUTTON (ps->m.l->spinbutton)->adjustment),
+		"value_changed", G_CALLBACK (gps_m_size_value_changed), ps);
+	g_signal_connect (
+		G_OBJECT (GTK_SPIN_BUTTON (ps->m.r->spinbutton)->adjustment),
+		"value_changed", G_CALLBACK (gps_m_size_value_changed), ps);
+	g_signal_connect (G_OBJECT (ps->m.t), "focus_in_event",
+		G_CALLBACK (lmargin_top_unit_activated), ps);
+	g_signal_connect (G_OBJECT (ps->m.t->spinbutton), "focus_out_event",
+		G_CALLBACK (lmargin_unit_deactivated), ps);
+	g_signal_connect (G_OBJECT (ps->m.l->spinbutton), "focus_in_event",
+		G_CALLBACK (lmargin_left_unit_activated), ps);
+	g_signal_connect (G_OBJECT (ps->m.l->spinbutton), "focus_out_event",
+		G_CALLBACK (lmargin_unit_deactivated), ps);
+	g_signal_connect (G_OBJECT (ps->m.r->spinbutton), "focus_in_event",
+		G_CALLBACK (lmargin_right_unit_activated), ps);
+	g_signal_connect (G_OBJECT (ps->m.r->spinbutton), "focus_out_event",
+		G_CALLBACK (lmargin_unit_deactivated), ps);
+	g_signal_connect (G_OBJECT (ps->m.b->spinbutton), "focus_in_event",
+		G_CALLBACK (lmargin_bottom_unit_activated), ps);
+	g_signal_connect (G_OBJECT (ps->m.b->spinbutton), "focus_out_event",
+		G_CALLBACK (lmargin_unit_deactivated), ps);
+
+	gtk_widget_show (GTK_WIDGET (ps->us));
 }
 
 GtkWidget *
